From 7ae5432fbfc0e923f2fe8dc62ff77afb8ecc80cf Mon Sep 17 00:00:00 2001
From: Thomas Bruederli <thomas@roundcube.net>
Date: Wed, 30 May 2012 04:42:27 -0400
Subject: [PATCH] Abbreviate long attachment file names with ellipsis (#1488499)

---
 program/include/rcube_message.php |  106 ++++++++++++++++++++++++++++++++++++++--------------
 1 files changed, 77 insertions(+), 29 deletions(-)

diff --git a/program/include/rcube_message.php b/program/include/rcube_message.php
index 8c61a4d..8f1432f 100644
--- a/program/include/rcube_message.php
+++ b/program/include/rcube_message.php
@@ -6,7 +6,10 @@
  |                                                                       |
  | This file is part of the Roundcube Webmail client                     |
  | Copyright (C) 2008-2010, The Roundcube Dev Team                       |
- | Licensed under the GNU GPL                                            |
+ |                                                                       |
+ | Licensed under the GNU General Public License version 3 or            |
+ | any later version with exceptions for skins & plugins.                |
+ | See the README file for a full license statement.                     |
  |                                                                       |
  | PURPOSE:                                                              |
  |   Logical representation of a mail message with all its data          |
@@ -37,11 +40,11 @@
     private $app;
 
     /**
-     * Instance of imap class
+     * Instance of storage class
      *
-     * @var rcube_imap
+     * @var rcube_storage
      */
-    private $imap;
+    private $storage;
 
     /**
      * Instance of mime class
@@ -70,7 +73,7 @@
      *
      * @param string $uid The message UID.
      *
-     * @see self::$app, self::$imap, self::$opt, self::$structure
+     * @see self::$app, self::$storage, self::$opt, self::$parts
      */
     function __construct($uid)
     {
@@ -161,22 +164,24 @@
     /**
      * Get content of a specific part of this message
      *
      * @param string $mime_id Part MIME-ID
      * @param resource $fp File pointer to save the message part
      * @return string Part content
      */
-    public function get_part_content($mime_id, $fp=NULL)
+    public function get_part_content($mime_id, $fp = null, $skip_charset_conv = false)
     {
         if ($part = $this->mime_parts[$mime_id]) {
             // stored in message structure (winmail/inline-uuencode)
-            if ($part->encoding == 'stream') {
+            if (!empty($part->body) || $part->encoding == 'stream') {
                 if ($fp) {
                     fwrite($fp, $part->body);
                 }
                 return $fp ? true : $part->body;
             }
             // get from IMAP
-            return $this->storage->get_message_part($this->uid, $mime_id, $part, NULL, $fp);
+            return $this->storage->get_message_part($this->uid, $mime_id, $part, NULL, $fp, $skip_charset_conv);
         } else
             return null;
     }
@@ -185,15 +190,36 @@
     /**
      * Determine if the message contains a HTML part
      *
+     * @param bool $recursive Enables checking in all levels of the structure
+     *
      * @return bool True if a HTML is available, False if not
      */
-    function has_html_part()
+    function has_html_part($recursive = true)
     {
         // check all message parts
-        foreach ($this->parts as $pid => $part) {
-            $mimetype = strtolower($part->ctype_primary . '/' . $part->ctype_secondary);
-            if ($mimetype == 'text/html')
+        foreach ($this->parts as $part) {
+            if ($part->mimetype == 'text/html') {
+                // Level check, we'll skip e.g. HTML attachments
+                if (!$recursive) {
+                    $level = explode('.', $part->mime_id);
+
+                    // Level too high
+                    if (count($level) > 2) {
+                        continue;
+                    }
+
+                    // HTML part can be on the lower level, if not...
+                    if (count($level) > 1) {
+                        // It can be an alternative or related message part
+                        $parent = $this->mime_parts[0];
+                        if ($parent->mimetype != 'multipart/alternative' && $parent->mimetype != 'multipart/related') {
+                            continue;
+                        }
+                    }
+                }
+
                 return true;
+            }
         }
 
         return false;
@@ -208,10 +234,9 @@
     function first_html_part()
     {
         // check all message parts
-        foreach ($this->mime_parts as $mime_id => $part) {
-            $mimetype = strtolower($part->ctype_primary . '/' . $part->ctype_secondary);
-            if ($mimetype == 'text/html') {
-                return $this->storage->get_message_part($this->uid, $mime_id, $part);
+        foreach ($this->mime_parts as $pid => $part) {
+            if ($part->mimetype == 'text/html') {
+                return $this->get_part_content($pid);
             }
         }
     }
@@ -231,13 +256,11 @@
 
         // check all message parts
         foreach ($this->mime_parts as $mime_id => $part) {
-            $mimetype = $part->ctype_primary . '/' . $part->ctype_secondary;
-
-            if ($mimetype == 'text/plain') {
-                return $this->storage->get_message_part($this->uid, $mime_id, $part);
+            if ($part->mimetype == 'text/plain') {
+                return $this->get_part_content($mime_id);
             }
-            else if ($mimetype == 'text/html') {
-                $out = $this->storage->get_message_part($this->uid, $mime_id, $part);
+            else if ($part->mimetype == 'text/html') {
+                $out = $this->get_part_content($mime_id);
 
                 // remove special chars encoding
                 $trans = array_flip(get_html_translation_table(HTML_ENTITIES));
@@ -255,7 +278,7 @@
 
 
     /**
-     * Raad the message structure returend by the IMAP server
+     * Read the message structure returend by the IMAP server
      * and build flat lists of content parts and attachments
      *
      * @param rcube_message_part $structure Message structure node
@@ -630,23 +653,29 @@
 
         $parts = array();
         // FIXME: line length is max.65?
-        $uu_regexp = '/begin [0-7]{3,4} ([^\n]+)\n(([\x21-\x7E]{0,65}\n)+)`\nend/s';
+        $uu_regexp = '/begin [0-7]{3,4} ([^\n]+)\n/s';
 
         if (preg_match_all($uu_regexp, $part->body, $matches, PREG_SET_ORDER)) {
-            // remove attachments bodies from the message body
-            $part->body = preg_replace($uu_regexp, '', $part->body);
             // update message content-type
             $part->ctype_primary   = 'multipart';
             $part->ctype_secondary = 'mixed';
             $part->mimetype        = $part->ctype_primary . '/' . $part->ctype_secondary;
+            $uu_endstring = "`\nend\n";
 
             // add attachments to the structure
             foreach ($matches as $pid => $att) {
+                $startpos = strpos($part->body, $att[1]) + strlen($att[1]) + 1; // "\n"
+                $endpos = strpos($part->body, $uu_endstring);
+                $filebody = substr($part->body, $startpos, $endpos-$startpos);
+
+                // remove attachments bodies from the message body
+                $part->body = substr_replace($part->body, "", $startpos, $endpos+strlen($uu_endstring)-$startpos);
+
                 $uupart = new rcube_message_part;
 
                 $uupart->filename = trim($att[1]);
                 $uupart->encoding = 'stream';
-                $uupart->body     = convert_uudecode($att[2]);
+                $uupart->body     = convert_uudecode($filebody);
                 $uupart->size     = strlen($uupart->body);
                 $uupart->mime_id  = 'uu.' . $part->mime_id . '.' . $pid;
 
@@ -657,8 +686,27 @@
                 $parts[] = $uupart;
                 unset($matches[$pid]);
             }
+
+            // remove attachments bodies from the message body
+            $part->body = preg_replace($uu_regexp, '', $part->body);
         }
 
         return $parts;
     }
+
+
+    /**
+     * Deprecated methods (to be removed)
+     */
+
+    public static function unfold_flowed($text)
+    {
+        return rcube_mime::unfold_flowed($text);
+    }
+
+    public static function format_flowed($text, $length = 72)
+    {
+        return rcube_mime::format_flowed($text, $length);
+    }
+
 }
@@ -161,22 +164,24 @@
      * Get content of a specific part of this message
      *
-     * @param string $mime_id Part MIME-ID
-     * @param resource $fp File pointer to save the message part
+     * @param string   $mime_id           Part MIME-ID
+     * @param resource $fp File           pointer to save the message part
+     * @param boolean  $skip_charset_conv Disables charset conversion
+     *
      * @return string Part content
      */

--
Gitblit v1.9.1